@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import difflib
 import os
@@ -28,6 +28,7 @@ import tempfile
 import time
 
 from bzrlib import (
+    branch as _mod_branch,
     bzrdir,
     commands,
     errors,
@@ -35,14 +36,14 @@ from bzrlib import (
     patiencediff,
     textfile,
     timestamp,
+    views,
     )
 """)
 
 from bzrlib.symbol_versioning import (
-        deprecated_function,
-        one_zero,
-        )
-from bzrlib.trace import mutter, warning
+    deprecated_function,
+    )
+from bzrlib.trace import mutter, note, warning
 
 
 # TODO: Rather than building a changeset object, we should probably
@@ -77,7 +78,7 @@ def internal_diff(old_filename, oldlines, new_filename, newlines, to_file,
     # both sequences are empty.
     if not oldlines and not newlines:
         return
-    
+
     if allow_binary is False:
         textfile.check_text_lines(oldlines)
         textfile.check_text_lines(newlines)
@@ -98,9 +99,6 @@ def internal_diff(old_filename, oldlines, new_filename, newlines, to_file,
         ud[2] = ud[2].replace('-1,0', '-0,0')
     elif not newlines:
         ud[2] = ud[2].replace('+1,0', '+0,0')
-    # work around for difflib emitting random spaces after the label
-    ud[0] = ud[0][:-2] + '\n'
-    ud[1] = ud[1][:-2] + '\n'
 
     for line in ud:
         to_file.write(line)
@@ -173,6 +171,11 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
 
         if not diff_opts:
             diff_opts = []
+        if sys.platform == 'win32':
+            # Popen doesn't do the proper encoding for external commands
+            # Since we are dealing with an ANSI api, use mbcs encoding
+            old_filename = old_filename.encode('mbcs')
+            new_filename = new_filename.encode('mbcs')
         diffcmd = ['gdiff',
                    '--label', old_filename,
                    old_abspath,
@@ -201,14 +204,14 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
             break
         else:
             diffcmd.append('-u')
-                  
+
         if diff_opts:
             diffcmd.extend(diff_opts)
 
         pipe = _spawn_external_diff(diffcmd, capture_errors=True)
         out,err = pipe.communicate()
         rc = pipe.returncode
-        
+
         # internal_diff() adds a trailing newline, add one here for consistency
         out += '\n'
         if rc == 2:
@@ -249,8 +252,8 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                 msg = 'signal %d' % (-rc)
             else:
                 msg = 'exit code %d' % rc
-                
-            raise errors.BzrError('external diff failed with %s; command: %r' 
+
+            raise errors.BzrError('external diff failed with %s; command: %r'
                                   % (rc, diffcmd))
 
 
@@ -274,84 +277,8 @@ def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                         new_abspath, e)
 
 
-@deprecated_function(one_zero)
-def diff_cmd_helper(tree, specific_files, external_diff_options, 
-                    old_revision_spec=None, new_revision_spec=None,
-                    revision_specs=None,
-                    old_label='a/', new_label='b/'):
-    """Helper for cmd_diff.
-
-    :param tree:
-        A WorkingTree
-
-    :param specific_files:
-        The specific files to compare, or None
-
-    :param external_diff_options:
-        If non-None, run an external diff, and pass it these options
-
-    :param old_revision_spec:
-        If None, use basis tree as old revision, otherwise use the tree for
-        the specified revision. 
-
-    :param new_revision_spec:
-        If None, use working tree as new revision, otherwise use the tree for
-        the specified revision.
-    
-    :param revision_specs: 
-        Zero, one or two RevisionSpecs from the command line, saying what revisions 
-        to compare.  This can be passed as an alternative to the old_revision_spec 
-        and new_revision_spec parameters.
-
-    The more general form is show_diff_trees(), where the caller
-    supplies any two trees.
-    """
-
-    # TODO: perhaps remove the old parameters old_revision_spec and
-    # new_revision_spec, since this is only really for use from cmd_diff and
-    # it now always passes through a sequence of revision_specs -- mbp
-    # 20061221
-
-    def spec_tree(spec):
-        if tree:
-            revision = spec.in_store(tree.branch)
-        else:
-            revision = spec.in_store(None)
-        revision_id = revision.rev_id
-        branch = revision.branch
-        return branch.repository.revision_tree(revision_id)
-
-    if revision_specs is not None:
-        assert (old_revision_spec is None
-                and new_revision_spec is None)
-        if len(revision_specs) > 0:
-            old_revision_spec = revision_specs[0]
-        if len(revision_specs) > 1:
-            new_revision_spec = revision_specs[1]
-
-    if old_revision_spec is None:
-        old_tree = tree.basis_tree()
-    else:
-        old_tree = spec_tree(old_revision_spec)
-
-    if (new_revision_spec is None
-        or new_revision_spec.spec is None):
-        new_tree = tree
-    else:
-        new_tree = spec_tree(new_revision_spec)
-
-    if new_tree is not tree:
-        extra_trees = (tree,)
-    else:
-        extra_trees = None
-
-    return show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
-                           external_diff_options,
-                           old_label=old_label, new_label=new_label,
-                           extra_trees=extra_trees)
-
-
-def _get_trees_to_diff(path_list, revision_specs, old_url, new_url):
+def _get_trees_to_diff(path_list, revision_specs, old_url, new_url,
+    apply_view=True):
     """Get the trees and specific files to diff given a list of paths.
 
     This method works out the trees to be diff'ed and the files of
@@ -368,6 +295,9 @@ def _get_trees_to_diff(path_list, revision_specs, old_url, new_url):
     :param new_url:
         The url of the new branch or tree. If None, the tree to use is
         taken from the first path, if any, or the current working tree.
+    :param apply_view:
+        if True and a view is set, apply the view or check that the paths
+        are within it
     :returns:
         a tuple of (old_tree, new_tree, specific_files, extra_trees) where
         extra_trees is a sequence of additional trees to search in for
@@ -407,6 +337,8 @@ def _get_trees_to_diff(path_list, revision_specs, old_url, new_url):
     working_tree, branch, relpath = \
         bzrdir.BzrDir.open_containing_tree_or_branch(old_url)
     if consider_relpath and relpath != '':
+        if working_tree is not None and apply_view:
+            views.check_path_in_view(working_tree, relpath)
         specific_files.append(relpath)
     old_tree = _get_tree_to_diff(old_revision_spec, working_tree, branch)
 
@@ -417,16 +349,30 @@ def _get_trees_to_diff(path_list, revision_specs, old_url, new_url):
         working_tree, branch, relpath = \
             bzrdir.BzrDir.open_containing_tree_or_branch(new_url)
         if consider_relpath and relpath != '':
+            if working_tree is not None and apply_view:
+                views.check_path_in_view(working_tree, relpath)
             specific_files.append(relpath)
     new_tree = _get_tree_to_diff(new_revision_spec, working_tree, branch,
         basis_is_default=working_tree is None)
 
     # Get the specific files (all files is None, no files is [])
     if make_paths_wt_relative and working_tree is not None:
-        other_paths = _relative_paths_in_tree(working_tree, other_paths)
+        try:
+            from bzrlib.builtins import safe_relpath_files
+            other_paths = safe_relpath_files(working_tree, other_paths,
+            apply_view=apply_view)
+        except errors.FileInWrongBranch:
+            raise errors.BzrCommandError("Files are in different branches")
     specific_files.extend(other_paths)
     if len(specific_files) == 0:
         specific_files = None
+        if (working_tree is not None and working_tree.supports_views()
+            and apply_view):
+            view_files = working_tree.views.lookup_view()
+            if view_files:
+                specific_files = view_files
+                view_str = views.view_display_str(view_files)
+                note("*** Ignoring files outside view. View is %s" % view_str)
 
     # Get extra trees that ought to be searched for file-ids
     extra_trees = None
@@ -434,7 +380,6 @@ def _get_trees_to_diff(path_list, revision_specs, old_url, new_url):
         extra_trees = (working_tree,)
     return old_tree, new_tree, specific_files, extra_trees
 
-
 def _get_tree_to_diff(spec, tree=None, branch=None, basis_is_default=True):
     if branch is None and tree is not None:
         branch = tree.branch
@@ -446,25 +391,7 @@ def _get_tree_to_diff(spec, tree=None, branch=None, basis_is_default=True):
                 return branch.basis_tree()
         else:
             return tree
-    revision = spec.in_store(branch)
-    revision_id = revision.rev_id
-    rev_branch = revision.branch
-    return rev_branch.repository.revision_tree(revision_id)
-
-
-def _relative_paths_in_tree(tree, paths):
-    """Get the relative paths within a working tree.
-
-    Each path may be either an absolute path or a path relative to the
-    current working directory.
-    """
-    result = []
-    for filename in paths:
-        try:
-            result.append(tree.relpath(osutils.dereference_path(filename)))
-        except errors.PathNotChild:
-            raise errors.BzrCommandError("Files are in different branches")
-    return result
+    return spec.as_tree(branch)
 
 
 def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
@@ -515,36 +442,15 @@ def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
 def _patch_header_date(tree, file_id, path):
     """Returns a timestamp suitable for use in a patch header."""
     mtime = tree.get_file_mtime(file_id, path)
-    assert mtime is not None, \
-        "got an mtime of None for file-id %s, path %s in tree %s" % (
-                file_id, path, tree)
     return timestamp.format_patch_date(mtime)
 
 
-def _raise_if_nonexistent(paths, old_tree, new_tree):
-    """Complain if paths are not in either inventory or tree.
-
-    It's OK with the files exist in either tree's inventory, or 
-    if they exist in the tree but are not versioned.
-    
-    This can be used by operations such as bzr status that can accept
-    unknown or ignored files.
-    """
-    mutter("check paths: %r", paths)
-    if not paths:
-        return
-    s = old_tree.filter_unversioned_files(paths)
-    s = new_tree.filter_unversioned_files(s)
-    s = [path for path in s if not new_tree.has_filename(path)]
-    if s:
-        raise errors.PathsDoNotExist(sorted(s))
-
-
-def get_prop_change(meta_modified):
-    if meta_modified:
-        return " (properties changed)"
+def get_executable_change(old_is_x, new_is_x):
+    descr = { True:"+x", False:"-x", None:"??" }
+    if old_is_x != new_is_x:
+        return ["%s to %s" % (descr[old_is_x], descr[new_is_x],)]
     else:
-        return  ""
+        return []
 
 
 class DiffPath(object):
@@ -719,9 +625,11 @@ class DiffText(DiffPath):
             return self.CANNOT_DIFF
         from_label = '%s%s\t%s' % (self.old_label, old_path, old_date)
         to_label = '%s%s\t%s' % (self.new_label, new_path, new_date)
-        return self.diff_text(from_file_id, to_file_id, from_label, to_label)
+        return self.diff_text(from_file_id, to_file_id, from_label, to_label,
+            old_path, new_path)
 
-    def diff_text(self, from_file_id, to_file_id, from_label, to_label):
+    def diff_text(self, from_file_id, to_file_id, from_label, to_label,
+        from_path=None, to_path=None):
         """Diff the content of given files in two trees
 
         :param from_file_id: The id of the file in the from tree.  If None,
@@ -729,15 +637,17 @@ class DiffText(DiffPath):
         :param to_file_id: The id of the file in the to tree.  This may refer
             to a different file from from_file_id.  If None,
             the file is not present in the to tree.
+        :param from_path: The path in the from tree or None if unknown.
+        :param to_path: The path in the to tree or None if unknown.
         """
-        def _get_text(tree, file_id):
+        def _get_text(tree, file_id, path):
             if file_id is not None:
-                return tree.get_file(file_id).readlines()
+                return tree.get_file(file_id, path).readlines()
             else:
                 return []
         try:
-            from_text = _get_text(self.old_tree, from_file_id)
-            to_text = _get_text(self.new_tree, to_file_id)
+            from_text = _get_text(self.old_tree, from_file_id, from_path)
+            to_text = _get_text(self.new_tree, to_file_id, to_path)
             self.text_differ(from_label, from_text, to_label, to_text,
                              self.to_file)
         except errors.BinaryFile:
@@ -753,7 +663,7 @@ class DiffFromTool(DiffPath):
                  path_encoding='utf-8'):
         DiffPath.__init__(self, old_tree, new_tree, to_file, path_encoding)
         self.command_template = command_template
-        self._root = tempfile.mkdtemp(prefix='bzr-diff-')
+        self._root = osutils.mkdtemp(prefix='bzr-diff-')
 
     @classmethod
     def from_string(klass, command_string, old_tree, new_tree, to_file,
@@ -788,8 +698,8 @@ class DiffFromTool(DiffPath):
         return proc.wait()
 
     def _try_symlink_root(self, tree, prefix):
-        if not (getattr(tree, 'abspath', None) is not None
-                and osutils.has_symlinks()):
+        if (getattr(tree, 'abspath', None) is None
+            or not osutils.host_os_dereferences_symlinks()):
             return False
         try:
             os.symlink(tree.abspath(''), osutils.pathjoin(self._root, prefix))
@@ -830,7 +740,12 @@ class DiffFromTool(DiffPath):
         return old_disk_path, new_disk_path
 
     def finish(self):
-        osutils.rmtree(self._root)
+        try:
+            osutils.rmtree(self._root)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                mutter("The temporary directory \"%s\" was not "
+                        "cleanly removed: %s." % (self._root, e))
 
     def diff(self, file_id, old_path, new_path, old_kind, new_kind):
         if (old_kind, new_kind) != ('file', 'file'):
@@ -904,7 +819,6 @@ class DiffTree(object):
         else:
             extra_factories = []
         if external_diff_options:
-            assert isinstance(external_diff_options, basestring)
             opts = external_diff_options.split()
             def diff_file(olab, olines, nlab, nlines, to_file):
                 external_diff(olab, olines, nlab, nlines, to_file, opts)
@@ -946,7 +860,9 @@ class DiffTree(object):
                 return path.encode(self.path_encoding, "replace")
         for (file_id, paths, changed_content, versioned, parent, name, kind,
              executable) in sorted(iterator, key=changes_key):
-            if parent == (None, None):
+            # The root does not get diffed, and items with no known kind (that
+            # is, missing) in both trees are skipped as well.
+            if parent == (None, None) or kind == (None, None):
                 continue
             oldpath, newpath = paths
             oldpath_encoded = get_encoded_path(paths[0])
@@ -954,7 +870,15 @@ class DiffTree(object):
             old_present = (kind[0] is not None and versioned[0])
             new_present = (kind[1] is not None and versioned[1])
             renamed = (parent[0], name[0]) != (parent[1], name[1])
-            prop_str = get_prop_change(executable[0] != executable[1])
+
+            properties_changed = []
+            properties_changed.extend(get_executable_change(executable[0], executable[1]))
+
+            if properties_changed:
+                prop_str = " (properties changed: %s)" % (", ".join(properties_changed),)
+            else:
+                prop_str = ""
+
             if (old_present, new_present) == (True, False):
                 self.to_file.write("=== removed %s '%s'\n" %
                                    (kind[0], oldpath_encoded))
@@ -972,7 +896,7 @@ class DiffTree(object):
                 self.to_file.write("=== modified %s '%s'%s\n" % (kind[0],
                                    newpath_encoded, prop_str))
             if changed_content:
-                self.diff(file_id, oldpath, newpath)
+                self._diff(file_id, oldpath, newpath, kind[0], kind[1])
                 has_changes = 1
             if renamed:
                 has_changes = 1
@@ -993,7 +917,10 @@ class DiffTree(object):
             new_kind = self.new_tree.kind(file_id)
         except (errors.NoSuchId, errors.NoSuchFile):
             new_kind = None
+        self._diff(file_id, old_path, new_path, old_kind, new_kind)
 
+
+    def _diff(self, file_id, old_path, new_path, old_kind, new_kind):
         result = DiffPath._diff_many(self.differs, file_id, old_path,
                                        new_path, old_kind, new_kind)
         if result is DiffPath.CANNOT_DIFF:
