@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """A collection of extra help information for using bzr.
 
@@ -126,7 +126,7 @@ def _help_on_topics(dummy):
 
     topics = topic_registry.keys()
     lmax = max(len(topic) for topic in topics)
-        
+
     out = []
     for topic in topics:
         summary = topic_registry.get_summary(topic)
@@ -149,14 +149,37 @@ def _help_on_revisionspec(name):
     import bzrlib.revisionspec
 
     out = []
-    out.append("Revision Identifiers\n")
-    out.append("A revision, or a range bound, can be one of the following.\n")
+    out.append(
+"""Revision Identifiers
+
+A revision identifier refers to a specific state of a branch's history. It can
+be a revision number, or a keyword followed by ':' and often other
+parameters. Some examples of identifiers are '3', 'last:1', 'before:yesterday'
+and 'submit:'.
+
+If 'REV1' and 'REV2' are revision identifiers, then 'REV1..REV2' denotes a
+revision range. Examples: '3647..3649', 'date:yesterday..-1' and
+'branch:/path/to/branch1/..branch:/branch2' (note that there are no quotes or
+spaces around the '..').
+
+Ranges are interpreted differently by different commands. To the "log" command,
+a range is a sequence of log messages, but to the "diff" command, the range
+denotes a change between revisions (and not a sequence of changes).  In
+addition, "log" considers a closed range whereas "diff" and "merge" consider it
+to be open-ended, that is, they include one end but not the other.  For example:
+"bzr log -r 3647..3649" shows the messages of revisions 3647, 3648 and 3649,
+while "bzr diff -r 3647..3649" includes the changes done in revisions 3648 and
+3649, but not 3647.
+
+The keywords used as revision selection methods are the following:
+""")
     details = []
-    details.append("\nFurther details are given below.\n")
+    details.append("\nIn addition, plugins can provide other keywords.")
+    details.append("\nA detailed description of each keyword is given below.\n")
 
     # The help text is indented 4 spaces - this re cleans that up below
     indent_re = re.compile(r'^    ', re.MULTILINE)
-    for i in bzrlib.revisionspec.SPEC_TYPES:
+    for prefix, i in bzrlib.revisionspec.revspec_registry.iteritems():
         doc = i.help_txt
         if doc == bzrlib.revisionspec.RevisionSpec.help_txt:
             summary = "N/A"
@@ -168,7 +191,7 @@ def _help_on_revisionspec(name):
             #doc = indent_re.sub('', doc)
             while (doc[-2:] == '\n\n' or doc[-1:] == ' '):
                 doc = doc[:-1]
-        
+
         # Note: The leading : here are HACKs to get reStructuredText
         # 'field' formatting - we know that the prefix ends in a ':'.
         out.append(":%s\n\t%s" % (i.prefix, summary))
@@ -184,7 +207,8 @@ def _help_on_transport(name):
     import textwrap
 
     def add_string(proto, help, maxl, prefix_width=20):
-       help_lines = textwrap.wrap(help, maxl - prefix_width)
+       help_lines = textwrap.wrap(help, maxl - prefix_width,
+            break_long_words=False)
        line_with_indent = '\n' + ' ' * prefix_width
        help_text = line_with_indent.join(help_lines)
        return "%-20s%s\n" % (proto, help_text)
@@ -241,6 +265,7 @@ Basic commands:
 
   bzr merge          pull in changes from another branch
   bzr commit         save some or all changes
+  bzr send           send changes via email
 
   bzr log            show history of changes
   bzr check          validate storage
@@ -275,30 +300,14 @@ command.  (e.g. "bzr --profile help").
 
 See doc/developers/profiling.txt for more information on profiling.
 A number of debug flags are also available to assist troubleshooting and
-development.
-
--Dauth         Trace authentication sections used.
--Derror        Instead of normal error handling, always print a traceback on
-               error.
--Devil         Capture call sites that do expensive or badly-scaling
-               operations.
--Dfetch        Trace history copying between repositories.
--Dhashcache    Log every time a working file is read to determine its hash.
--Dhooks        Trace hook execution.
--Dhpss         Trace smart protocol requests and responses.
--Dhttp         Trace http connections, requests and responses
--Dindex        Trace major index operations.
--Dknit         Trace knit operations.
--Dlock         Trace when lockdir locks are taken or released.
--Dmerge        Emit information for debugging merges.
--Dpack         Emit information about pack operations.
+development.  See `bzr help debug-flags`.
 """
 
 _standard_options = \
 """Standard Options
 
 Standard options are legal for all commands.
-      
+
 --help, -h     Show help message.
 --verbose, -v  Display more information.
 --quiet, -q    Only display errors and warnings.
@@ -349,14 +358,14 @@ don't have a local branch, then you cannot commit locally.
 Lightweight checkouts work best when you have fast reliable access to the
 master branch. This means that if the master branch is on the same disk or LAN
 a lightweight checkout will be faster than a heavyweight one for any commands
-that modify the revision history (as only one copy branch needs to be updated).
-Heavyweight checkouts will generally be faster for any command that uses the
-history but does not change it, but if the master branch is on the same disk
-then there wont be a noticeable difference.
+that modify the revision history (as only one copy of the branch needs to
+be updated). Heavyweight checkouts will generally be faster for any command
+that uses the history but does not change it, but if the master branch is on
+the same disk then there won't be a noticeable difference.
 
 Another possible use for a checkout is to use it with a treeless repository
 containing your branches, where you maintain only one working tree by
-switching the master branch that the checkout points to when you want to 
+switching the master branch that the checkout points to when you want to
 work on a different branch.
 
 Obviously to commit on a checkout you need to be able to write to the master
@@ -377,7 +386,7 @@ Related commands::
               checkout
   update      Pull any changes in the master branch in to your checkout
   commit      Make a commit that is sent to the master branch. If you have
-              a heavy checkout then the --local option will commit to the 
+              a heavy checkout then the --local option will commit to the
               checkout without sending the commit to the master
   bind        Change the master branch that the commits in the checkout will
               be sent to
@@ -393,7 +402,7 @@ a repository associated with every branch.
 
 Repositories are a form of database. Bzr will usually maintain this for
 good performance automatically, but in some situations (e.g. when doing
-very many commits in a short time period) you may want to ask bzr to 
+very many commits in a short time period) you may want to ask bzr to
 optimise the database indices. This can be done by the 'bzr pack' command.
 
 By default just running 'bzr init' will create a repository within the new
@@ -480,8 +489,8 @@ repository (a shared repository). Branches can be copied and merged.
 
 Related commands::
 
-  init    Make a directory into a versioned branch.
-  branch  Create a new copy of a branch.
+  init    Change a directory into a versioned branch.
+  branch  Create a new branch that is a copy of an existing branch.
   merge   Perform a three-way merge.
 """
 
@@ -516,6 +525,7 @@ Column 1 - versioning/renames::
   - File unversioned
   R File renamed
   ? File unknown
+  X File nonexistent (and unknown to bzr)
   C File has conflicts
   P Entry for a pending merge (not a file)
 
@@ -579,8 +589,9 @@ _criss_cross = \
 A criss-cross in the branch history can cause the default merge technique
 to emit more conflicts than would normally be expected.
 
-If you encounter criss-crosses, you can use merge --weave instead, which
-should provide a much better result.
+In complex merge cases, ``bzr merge --lca`` or ``bzr merge --weave`` may give
+better results.  You may wish to ``bzr revert`` the working tree and merge
+again.  Alternatively, use ``bzr remerge`` on particular conflicted files.
 
 Criss-crosses occur in a branch's history if two branches merge the same thing
 and then merge one another, or if two branches merge one another at the same
@@ -600,7 +611,58 @@ are emitted.
 
 The ``weave`` merge type is not affected by this problem because it uses
 line-origin detection instead of a basis revision to determine the cause of
-differences."""
+differences.
+"""
+
+_branches_out_of_sync = """Branches out of sync
+
+When reconfiguring a checkout, tree or branch into a lightweight checkout,
+a local branch must be destroyed.  (For checkouts, this is the local branch
+that serves primarily as a cache.)  If the branch-to-be-destroyed does not
+have the same last revision as the new reference branch for the lightweight
+checkout, data could be lost, so Bazaar refuses.
+
+How you deal with this depends on *why* the branches are out of sync.
+
+If you have a checkout and have done local commits, you can get back in sync
+by running "bzr update" (and possibly "bzr commit").
+
+If you have a branch and the remote branch is out-of-date, you can push
+the local changes using "bzr push".  If the local branch is out of date, you
+can do "bzr pull".  If both branches have had changes, you can merge, commit
+and then push your changes.  If you decide that some of the changes aren't
+useful, you can "push --overwrite" or "pull --overwrite" instead.
+"""
+
+
+_storage_formats = \
+"""Storage Formats
+
+To ensure that older clients do not access data incorrectly,
+Bazaar's policy is to introduce a new storage format whenever
+new features requiring new metadata are added. New storage
+formats may also be introduced to improve performance and
+scalability.
+
+The newest format, 2a, is highly recommended. If your
+project is not using 2a, then you should suggest to the
+project owner to upgrade.
+
+
+Note: Some of the older formats have two variants:
+a plain one and a rich-root one. The latter include an additional
+field about the root of the tree. There is no performance cost
+for using a rich-root format but you cannot easily merge changes
+from a rich-root format into a plain format. As a consequence,
+moving a project to a rich-root format takes some co-ordination
+in that all contributors need to upgrade their repositories
+around the same time. 2a and all future formats will be
+implicitly rich-root.
+
+See ``bzr help current-formats`` for the complete list of
+currently supported formats. See ``bzr help other-formats`` for
+descriptions of any available experimental and deprecated formats.
+"""
 
 
 # Register help topics
@@ -608,10 +670,18 @@ topic_registry.register("revisionspec", _help_on_revisionspec,
                         "Explain how to use --revision")
 topic_registry.register('basic', _basic_help, "Basic commands", SECT_HIDDEN)
 topic_registry.register('topics', _help_on_topics, "Topics list", SECT_HIDDEN)
-def get_format_topic(topic):
+def get_current_formats_topic(topic):
     from bzrlib import bzrdir
-    return "Storage Formats\n\n" + bzrdir.format_registry.help_topic(topic)
-topic_registry.register('formats', get_format_topic, 'Directory formats')
+    return "Current Storage Formats\n\n" + \
+        bzrdir.format_registry.help_topic(topic)
+def get_other_formats_topic(topic):
+    from bzrlib import bzrdir
+    return "Other Storage Formats\n\n" + \
+        bzrdir.format_registry.help_topic(topic)
+topic_registry.register('current-formats', get_current_formats_topic,
+    'Current storage formats')
+topic_registry.register('other-formats', get_other_formats_topic,
+    'Experimental and deprecated storage formats')
 topic_registry.register('standard-options', _standard_options,
                         'Options that can be used with any command')
 topic_registry.register('global-options', _global_options,
@@ -622,23 +692,31 @@ topic_registry.register('status-flags', _status_flags,
                         "Help on status flags")
 def get_bugs_topic(topic):
     from bzrlib import bugtracker
-    return ("Bug Tracker Settings\n\n" + 
+    return ("Bug Tracker Settings\n\n" +
         bugtracker.tracker_registry.help_topic(topic))
 topic_registry.register('bugs', get_bugs_topic, 'Bug tracker settings')
 topic_registry.register('env-variables', _env_variables,
                         'Environment variable names and values')
 topic_registry.register('files', _files,
                         'Information on configuration and log files')
+topic_registry.register_lazy('hooks', 'bzrlib.hooks', 'hooks_help_text',
+                        'Points at which custom processing can be added')
 
-# Load some of the help topics from files
+# Load some of the help topics from files. Note that topics which reproduce API
+# details will tend to skew (quickly usually!) so please seek other solutions
+# for such things.
 topic_registry.register('authentication', _load_from_file,
                         'Information on configuring authentication')
 topic_registry.register('configuration', _load_from_file,
                         'Details on the configuration settings available')
 topic_registry.register('conflicts', _load_from_file,
                         'Types of conflicts and what to do about them')
-topic_registry.register('hooks', _load_from_file,
-                        'Points at which custom processing can be added')
+topic_registry.register('debug-flags', _load_from_file,
+                        'Options to show or record debug information')
+topic_registry.register('log-formats', _load_from_file,
+                        'Details on the logging formats available')
+topic_registry.register('diverged-branches', _load_from_file,
+                        'How to fix diverged branches')
 
 
 # Register concept topics.
@@ -649,9 +727,24 @@ topic_registry.register('branches', _branches,
                         'Information on what a branch is', SECT_CONCEPT)
 topic_registry.register('checkouts', _checkouts,
                         'Information on what a checkout is', SECT_CONCEPT)
+topic_registry.register('content-filters', _load_from_file,
+                        'Conversion of content into/from working trees',
+                        SECT_CONCEPT)
+topic_registry.register('eol', _load_from_file,
+                        'Information on end-of-line handling',
+                        SECT_CONCEPT)
+topic_registry.register('formats', _storage_formats,
+                        'Information on choosing a storage format',
+                        SECT_CONCEPT)
+topic_registry.register('patterns', _load_from_file,
+                        'Information on the pattern syntax',
+                        SECT_CONCEPT)
 topic_registry.register('repositories', _repositories,
                         'Basic information on shared repositories.',
                         SECT_CONCEPT)
+topic_registry.register('rules', _load_from_file,
+                        'Information on defining rule-based preferences',
+                        SECT_CONCEPT)
 topic_registry.register('standalone-trees', _standalone_trees,
                         'Information on what a standalone tree is',
                         SECT_CONCEPT)
@@ -659,6 +752,9 @@ topic_registry.register('working-trees', _working_trees,
                         'Information on working trees', SECT_CONCEPT)
 topic_registry.register('criss-cross', _criss_cross,
                         'Information on criss-cross merging', SECT_CONCEPT)
+topic_registry.register('sync-for-reconfigure', _branches_out_of_sync,
+                        'Steps to resolve "out-of-sync" when reconfiguring',
+                        SECT_CONCEPT)
 
 
 class HelpTopicIndex(object):
@@ -705,7 +801,7 @@ class RegisteredTopic(object):
             returned instead of plain text.
         """
         result = topic_registry.get_detail(self.topic)
-        # there is code duplicated here and in bzrlib/plugin.py's 
+        # there is code duplicated here and in bzrlib/plugin.py's
         # matching Topic code. This should probably be factored in
         # to a helper function and a common base class.
         if additional_see_also is not None:
