@@ -1,5 +1,5 @@
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -12,10 +12,11 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 __all__ = ['show_bzrdir_info']
 
+from cStringIO import StringIO
 import os
 import time
 import sys
@@ -24,14 +25,13 @@ from bzrlib import (
     bzrdir,
     diff,
     errors,
+    hooks as _mod_hooks,
     osutils,
     urlutils,
     )
 from bzrlib.errors import (NoWorkingTree, NotBranchError,
                            NoRepositoryPresent, NotLocalUrl)
 from bzrlib.missing import find_unmerged
-from bzrlib.symbol_versioning import (deprecated_function,
-        zero_eighteen)
 
 
 def plural(n, base='', pl=None):
@@ -143,6 +143,11 @@ def _gather_related_branches(branch):
     locs.add_url('push branch', branch.get_push_location())
     locs.add_url('parent branch', branch.get_parent())
     locs.add_url('submit branch', branch.get_submit_branch())
+    try:
+        locs.add_url('stacked on', branch.get_stacked_on_url())
+    except (errors.UnstackableBranchFormat, errors.UnstackableRepositoryFormat,
+        errors.NotStacked):
+        pass
     return locs
 
 
@@ -258,7 +263,7 @@ def _show_working_stats(working, outfile):
 
     dir_cnt = 0
     for file_id in work_inv:
-        if (work_inv.get_file_kind(file_id) == 'directory' and 
+        if (work_inv.get_file_kind(file_id) == 'directory' and
             not work_inv.is_root(file_id)):
             dir_cnt += 1
     outfile.write('  %8d versioned %s\n' % (dir_cnt,
@@ -296,16 +301,20 @@ def _show_repository_info(repository, outfile):
             'the repository.\n')
 
 
-def _show_repository_stats(stats, outfile):
+def _show_repository_stats(repository, stats, outfile):
     """Show statistics about a repository."""
-    if 'revisions' in stats or 'size' in stats:
-        outfile.write('\n')
-        outfile.write('Repository:\n')
+    f = StringIO()
     if 'revisions' in stats:
         revisions = stats['revisions']
-        outfile.write('  %8d revision%s\n' % (revisions, plural(revisions)))
+        f.write('  %8d revision%s\n' % (revisions, plural(revisions)))
     if 'size' in stats:
-        outfile.write('  %8d KiB\n' % (stats['size']/1024))
+        f.write('  %8d KiB\n' % (stats['size']/1024))
+    for hook in hooks['repository']:
+        hook(repository, stats, f)
+    if f.getvalue() != "":
+        outfile.write('\n')
+        outfile.write('Repository:\n')
+        outfile.write(f.getvalue())
 
 
 def show_bzrdir_info(a_bzrdir, verbose=False, outfile=None):
@@ -373,12 +382,13 @@ def show_component_info(control, repository, branch=None, working=None,
     elif branch is not None:
         _show_missing_revisions_branch(branch, outfile)
     if branch is not None:
-        stats = _show_branch_stats(branch, verbose==2, outfile)
+        show_committers = verbose >= 2
+        stats = _show_branch_stats(branch, show_committers, outfile)
     else:
         stats = repository.gather_stats()
     if branch is None and working is None:
         _show_repository_info(repository, outfile)
-    _show_repository_stats(stats, outfile)
+    _show_repository_stats(repository, stats, outfile)
 
 
 def describe_layout(repository=None, branch=None, tree=None):
@@ -468,3 +478,17 @@ def describe_format(control, repository, branch, tree):
         # do.
         candidates = new_candidates
     return ' or '.join(candidates)
+
+
+class InfoHooks(_mod_hooks.Hooks):
+    """Hooks for the info command."""
+
+    def __init__(self):
+        self.create_hook(_mod_hooks.HookPoint('repository',
+            "Invoked when displaying the statistics for a repository. "
+            "repository is called with a statistics dictionary as returned "
+            "by the repository and a file-like object to write to.", (1, 15), 
+            None))
+
+
+hooks = InfoHooks()
