@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import base64
 import os
@@ -92,13 +92,15 @@ class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
         self.connected_host = host
         if self.expect_auth:
             auth_hdrs = [v for k,v in http_headers if k == 'Authorization']
-            assert len(auth_hdrs) == 1
+            if len(auth_hdrs) != 1:
+                raise AssertionError("multiple auth headers: %r"
+                    % (auth_hdrs,))
             authinfo = auth_hdrs[0]
             expected_auth = 'testuser@launchpad.net:testpassword'
             test.assertEquals(authinfo,
                     'Basic ' + base64.encodestring(expected_auth).strip())
-        else:
-            assert not http_headers
+        elif http_headers:
+            raise AssertionError()
         return InstrumentedXMLRPCConnection(test)
 
     def send_request(self, connection, handler_path, request_body):
@@ -115,8 +117,9 @@ class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
 
     def send_content(self, conn, request_body):
         unpacked, method = xmlrpclib.loads(request_body)
-        assert None not in unpacked, \
-                "xmlrpc result %r shouldn't contain None" % (unpacked,)
+        if None in unpacked:
+            raise AssertionError(
+                "xmlrpc result %r shouldn't contain None" % (unpacked,))
         self.sent_params = unpacked
 
 
@@ -332,12 +335,14 @@ class TestGatherUserCredentials(tests.TestCaseInTempDir):
         g_conf = config.GlobalConfig()
         g_conf.set_user_option('email', 'Test User <test@user.com>')
         stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
         ui.ui_factory = tests.TestUIFactory(stdin='userpass\n',
-                                            stdout=stdout)
+                                            stdout=stdout, stderr=stderr)
         self.assertIs(None, service.registrant_password)
         service.gather_user_credentials()
         self.assertEqual('test@user.com', service.registrant_email)
         self.assertEqual('userpass', service.registrant_password)
-        self.assertContainsRe(stdout.getvalue(),
+        self.assertEquals('', stdout.getvalue())
+        self.assertContainsRe(stderr.getvalue(),
                              'launchpad.net password for test@user\\.com')
 
