@@ -13,19 +13,21 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Black-box tests for bzr.
 
 These check that it behaves properly when it's invoked through the regular
-command-line interface. This doesn't actually run a new interpreter but 
+command-line interface. This doesn't actually run a new interpreter but
 rather starts again from the run_bzr function.
 """
 
 
 import os
 
+from bzrlib.branch import Branch
+from bzrlib.config import extract_email_address
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -144,9 +146,81 @@ class TestAnnotate(TestCaseWithTransport):
                                 retcode=3)
         self.assertEqual('', out)
         self.assertEqual('bzr: ERROR: bzr annotate --revision takes'
-                         ' exactly 1 argument\n',
+                         ' exactly one revision identifier\n',
                          err)
 
+
+class TestSimpleAnnotate(TestCaseWithTransport):
+    """Annotate tests with no complex setup."""
+
+    def _setup_edited_file(self):
+        """Create a tree with a locally edited file."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('file', 'foo\ngam\n')])
+        tree.add('file')
+        tree.commit('add file', committer="test@host", rev_id="rev1")
+        self.build_tree_contents([('file', 'foo\nbar\ngam\n')])
+        tree.branch.get_config().set_user_option('email', 'current@host2')
+
+    def test_annotate_edited_file(self):
+        tree = self._setup_edited_file()
+        out, err = self.run_bzr('annotate file')
+        self.assertEqual(
+            '1   test@ho | foo\n'
+            '2?  current | bar\n'
+            '1   test@ho | gam\n',
+            out)
+
+    def test_annotate_edited_file_show_ids(self):
+        tree = self._setup_edited_file()
+        out, err = self.run_bzr('annotate file --show-ids')
+        self.assertEqual(
+            '    rev1 | foo\n'
+            'current: | bar\n'
+            '    rev1 | gam\n',
+            out)
+
+    def _create_merged_file(self):
+        """Create a file with a pending merge and local edit."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('file', 'foo\ngam\n')])
+        tree.add('file')
+        tree.commit('add file', rev_id="rev1", committer="test@host")
+        # right side
+        self.build_tree_contents([('file', 'foo\nbar\ngam\n')])
+        tree.commit("right", rev_id="rev1.1.1", committer="test@host")
+        tree.pull(tree.branch, True, "rev1")
+        # left side
+        self.build_tree_contents([('file', 'foo\nbaz\ngam\n')])
+        tree.commit("left", rev_id="rev2", committer="test@host")
+        # merge
+        tree.merge_from_branch(tree.branch, "rev1.1.1")
+        # edit the file to be 'resolved' and have a further local edit
+        self.build_tree_contents([('file', 'local\nfoo\nbar\nbaz\ngam\n')])
+
+    def test_annotated_edited_merged_file_revnos(self):
+        self._create_merged_file()
+        out, err = self.run_bzr('annotate file')
+        email = extract_email_address(Branch.open('.').get_config().username())
+        self.assertEqual(
+            '3?    %-7s | local\n'
+            '1     test@ho | foo\n'
+            '1.1.1 test@ho | bar\n'
+            '2     test@ho | baz\n'
+            '1     test@ho | gam\n' % email[:7],
+            out)
+
+    def test_annotated_edited_merged_file_ids(self):
+        self._create_merged_file()
+        out, err = self.run_bzr('annotate file --show-ids')
+        self.assertEqual(
+            'current: | local\n'
+            '    rev1 | foo\n'
+            'rev1.1.1 | bar\n'
+            '    rev2 | baz\n'
+            '    rev1 | gam\n',
+            out)
+
     def test_annotate_empty_file(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/empty', '')])
@@ -157,6 +231,16 @@ class TestAnnotate(TestCaseWithTransport):
         out, err = self.run_bzr('annotate empty')
         self.assertEqual('', out)
 
+    def test_annotate_empty_file_show_ids(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/empty', '')])
+        tree.add('empty')
+        tree.commit('add empty file')
+
+        os.chdir('tree')
+        out, err = self.run_bzr(['annotate', '--show-ids', 'empty'])
+        self.assertEqual('', out)
+
     def test_annotate_nonexistant_file(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/file'])
