@@ -13,12 +13,13 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Foundation SSH support for SFTP and smart server."""
 
 import errno
 import getpass
+import logging
 import os
 import socket
 import subprocess
@@ -122,12 +123,15 @@ class SSHVendorManager(object):
         elif 'SSH Secure Shell' in version:
             trace.mutter('ssh implementation is SSH Corp.')
             vendor = SSHCorpSubprocessVendor()
-        elif 'plink' in version and args[0] == 'plink':
-            # Checking if "plink" was the executed argument as Windows
-            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
-            # version.  See https://bugs.launchpad.net/bzr/+bug/107155
-            trace.mutter("ssh implementation is Putty's plink.")
-            vendor = PLinkSubprocessVendor()
+        # Auto-detect of plink vendor disabled, on Windows recommended
+        # default ssh-client is paramiko
+        # see https://bugs.launchpad.net/bugs/414743
+        #~elif 'plink' in version and args[0] == 'plink':
+        #~    # Checking if "plink" was the executed argument as Windows
+        #~    # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+        #~    # version.  See https://bugs.launchpad.net/bzr/+bug/107155
+        #~    trace.mutter("ssh implementation is Putty's plink.")
+        #~    vendor = PLinkSubprocessVendor()
         return vendor
 
     def _get_vendor_by_inspection(self):
@@ -172,19 +176,34 @@ def _ignore_sigint():
     signal.signal(signal.SIGINT, signal.SIG_IGN)
 
 
-class LoopbackSFTP(object):
+class SocketAsChannelAdapter(object):
     """Simple wrapper for a socket that pretends to be a paramiko Channel."""
 
     def __init__(self, sock):
         self.__socket = sock
 
+    def get_name(self):
+        return "bzr SocketAsChannelAdapter"
+
     def send(self, data):
         return self.__socket.send(data)
 
     def recv(self, n):
-        return self.__socket.recv(n)
+        try:
+            return self.__socket.recv(n)
+        except socket.error, e:
+            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, errno.ECONNABORTED,
+                             errno.EBADF):
+                # Connection has closed.  Paramiko expects an empty string in
+                # this case, not an exception.
+                return ''
+            raise
 
     def recv_ready(self):
+        # TODO: jam 20051215 this function is necessary to support the
+        # pipelined() function. In reality, it probably should use
+        # poll() or select() to actually return if there is data
+        # available, otherwise we probably don't get any benefit
         return True
 
     def close(self):
@@ -196,7 +215,7 @@ class SSHVendor(object):
 
     def connect_sftp(self, username, password, host, port):
         """Make an SSH connection, and return an SFTPClient.
-        
+
         :param username: an ascii string
         :param password: an ascii string
         :param host: a host name as an ascii string
@@ -211,7 +230,7 @@ class SSHVendor(object):
 
     def connect_ssh(self, username, password, host, port, command):
         """Make an SSH connection.
-        
+
         :returns: something with a `close` method, and a `get_filelike_channels`
             method that returns a pair of (read, write) filelike objects.
         """
@@ -237,7 +256,7 @@ class LoopbackVendor(SSHVendor):
             sock.connect((host, port))
         except socket.error, e:
             self._raise_connection_error(host, port=port, orig_error=e)
-        return SFTPClient(LoopbackSFTP(sock))
+        return SFTPClient(SocketAsChannelAdapter(sock))
 
 register_ssh_vendor('loopback', LoopbackVendor())
 
@@ -347,7 +366,7 @@ class SubprocessVendor(SSHVendor):
             argv = self._get_vendor_specific_argv(username, host, port,
                                                   subsystem='sftp')
             sock = self._connect(argv)
-            return SFTPClient(sock)
+            return SFTPClient(SocketAsChannelAdapter(sock))
         except _sftp_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
         except (OSError, IOError), e:
@@ -376,7 +395,7 @@ class SubprocessVendor(SSHVendor):
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
         """Returns the argument list to run the subprocess with.
-        
+
         Exactly one of 'subsystem' and 'command' must be specified.
         """
         raise NotImplementedError(self._get_vendor_specific_argv)
@@ -387,11 +406,6 @@ class OpenSSHSubprocessVendor(SubprocessVendor):
 
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        assert subsystem is not None or command is not None, (
-            'Must specify a command or subsystem')
-        if subsystem is not None:
-            assert command is None, (
-                'subsystem and command are mutually exclusive')
         args = ['ssh',
                 '-oForwardX11=no', '-oForwardAgent=no',
                 '-oClearAllForwardings=yes', '-oProtocol=2',
@@ -414,11 +428,6 @@ class SSHCorpSubprocessVendor(SubprocessVendor):
 
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        assert subsystem is not None or command is not None, (
-            'Must specify a command or subsystem')
-        if subsystem is not None:
-            assert command is None, (
-                'subsystem and command are mutually exclusive')
         args = ['ssh', '-x']
         if port is not None:
             args.extend(['-p', str(port)])
@@ -438,11 +447,6 @@ class PLinkSubprocessVendor(SubprocessVendor):
 
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
                                   command=None):
-        assert subsystem is not None or command is not None, (
-            'Must specify a command or subsystem')
-        if subsystem is not None:
-            assert command is None, (
-                'subsystem and command are mutually exclusive')
         args = ['plink', '-x', '-a', '-ssh', '-2', '-batch']
         if port is not None:
             args.extend(['-P', str(port)])
@@ -458,19 +462,12 @@ register_ssh_vendor('plink', PLinkSubprocessVendor())
 
 
 def _paramiko_auth(username, password, host, port, paramiko_transport):
-    # paramiko requires a username, but it might be none if nothing was supplied
-    # use the local username, just in case.
-    # We don't override username, because if we aren't using paramiko,
-    # the username might be specified in ~/.ssh/config and we don't want to
-    # force it to something else
-    # Also, it would mess up the self.relpath() functionality
     auth = config.AuthenticationConfig()
+    # paramiko requires a username, but it might be none if nothing was
+    # supplied.  If so, use the local username.
     if username is None:
-        username = auth.get_user('ssh', host, port=port)
-        if username is None:
-            # Default to local user
-            username = getpass.getuser()
-
+        username = auth.get_user('ssh', host, port=port,
+                                 default=getpass.getuser())
     if _use_ssh_agent:
         agent = paramiko.Agent()
         for key in agent.get_keys():
@@ -488,6 +485,39 @@ def _paramiko_auth(username, password, host, port, paramiko_transport):
     if _try_pkey_auth(paramiko_transport, paramiko.DSSKey, username, 'id_dsa'):
         return
 
+    # If we have gotten this far, we are about to try for passwords, do an
+    # auth_none check to see if it is even supported.
+    supported_auth_types = []
+    try:
+        # Note that with paramiko <1.7.5 this logs an INFO message:
+        #    Authentication type (none) not permitted.
+        # So we explicitly disable the logging level for this action
+        old_level = paramiko_transport.logger.level
+        paramiko_transport.logger.setLevel(logging.WARNING)
+        try:
+            paramiko_transport.auth_none(username)
+        finally:
+            paramiko_transport.logger.setLevel(old_level)
+    except paramiko.BadAuthenticationType, e:
+        # Supported methods are in the exception
+        supported_auth_types = e.allowed_types
+    except paramiko.SSHException, e:
+        # Don't know what happened, but just ignore it
+        pass
+    # We treat 'keyboard-interactive' and 'password' auth methods identically,
+    # because Paramiko's auth_password method will automatically try
+    # 'keyboard-interactive' auth (using the password as the response) if
+    # 'password' auth is not available.  Apparently some Debian and Gentoo
+    # OpenSSH servers require this.
+    # XXX: It's possible for a server to require keyboard-interactive auth that
+    # requires something other than a single password, but we currently don't
+    # support that.
+    if ('password' not in supported_auth_types and
+        'keyboard-interactive' not in supported_auth_types):
+        raise errors.ConnectionError('Unable to authenticate to SSH host as'
+            '\n  %s@%s\nsupported auth types: %s'
+            % (username, host, supported_auth_types))
+
     if password:
         try:
             paramiko_transport.auth_password(username, password)
@@ -497,11 +527,17 @@ def _paramiko_auth(username, password, host, port, paramiko_transport):
 
     # give up and ask for a password
     password = auth.get_password('ssh', host, username, port=port)
-    try:
-        paramiko_transport.auth_password(username, password)
-    except paramiko.SSHException, e:
-        raise errors.ConnectionError(
-            'Unable to authenticate to SSH host as %s@%s' % (username, host), e)
+    # get_password can still return None, which means we should not prompt
+    if password is not None:
+        try:
+            paramiko_transport.auth_password(username, password)
+        except paramiko.SSHException, e:
+            raise errors.ConnectionError(
+                'Unable to authenticate to SSH host as'
+                '\n  %s@%s\n' % (username, host), e)
+    else:
+        raise errors.ConnectionError('Unable to authenticate to SSH host as'
+                                     '  %s@%s' % (username, host))
 
 
 def _try_pkey_auth(paramiko_transport, pkey_class, username, filename):
@@ -569,11 +605,11 @@ def save_host_keys():
 def os_specific_subprocess_params():
     """Get O/S specific subprocess parameters."""
     if sys.platform == 'win32':
-        # setting the process group and closing fds is not supported on 
+        # setting the process group and closing fds is not supported on
         # win32
         return {}
     else:
-        # We close fds other than the pipes as the child process does not need 
+        # We close fds other than the pipes as the child process does not need
         # them to be open.
         #
         # We also set the child process to ignore SIGINT.  Normally the signal
@@ -581,7 +617,7 @@ def os_specific_subprocess_params():
         # this causes it to be seen only by bzr and not by ssh.  Python will
         # generate a KeyboardInterrupt in bzr, and we will then have a chance
         # to release locks or do other cleanup over ssh before the connection
-        # goes away.  
+        # goes away.
         # <https://launchpad.net/products/bzr/+bug/5987>
         #
         # Running it in a separate process group is not good because then it
@@ -601,13 +637,6 @@ class SSHSubprocess(object):
     def send(self, data):
         return os.write(self.proc.stdin.fileno(), data)
 
-    def recv_ready(self):
-        # TODO: jam 20051215 this function is necessary to support the
-        # pipelined() function. In reality, it probably should use
-        # poll() or select() to actually return if there is data
-        # available, otherwise we probably don't get any benefit
-        return True
-
     def recv(self, count):
         return os.read(self.proc.stdout.fileno(), count)
 
