@@ -1,6 +1,4 @@
-# Bazaar -- distributed version control
-#
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """A collection of function for handling URL operations."""
 
@@ -26,6 +24,7 @@ from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from posixpath import split as _posix_split, normpath as _posix_normpath
 import urllib
+import urlparse
 
 from bzrlib import (
     errors,
@@ -67,18 +66,18 @@ def escape(relpath):
         relpath = relpath.encode('utf-8')
     # After quoting and encoding, the path should be perfectly
     # safe as a plain ASCII string, str() just enforces this
-    return str(urllib.quote(relpath))
+    return str(urllib.quote(relpath, safe='/~'))
 
 
 def file_relpath(base, path):
     """Compute just the relative sub-portion of a url
-    
+
     This assumes that both paths are already fully specified file:// URLs.
     """
-    assert len(base) >= MIN_ABS_FILEURL_LENGTH, ('Length of base must be equal or'
-        ' exceed the platform minimum url length (which is %d)' % 
-        MIN_ABS_FILEURL_LENGTH)
-
+    if len(base) < MIN_ABS_FILEURL_LENGTH:
+        raise ValueError('Length of base (%r) must equal or'
+            ' exceed the platform minimum url length (which is %d)' %
+            (base, MIN_ABS_FILEURL_LENGTH))
     base = local_path_from_url(base)
     path = local_path_from_url(path)
     return escape(osutils.relpath(base, path))
@@ -117,11 +116,13 @@ def join(base, *args):
         join('http://foo', 'bar') => 'http://foo/bar'
         join('http://foo', 'bar', '../baz') => 'http://foo/baz'
     """
-    m = _url_scheme_re.match(base)
+    if not args:
+        return base
+    match = _url_scheme_re.match(base)
     scheme = None
-    if m:
-        scheme = m.group('scheme')
-        path = m.group('path').split('/')
+    if match:
+        scheme = match.group('scheme')
+        path = match.group('path').split('/')
         if path[-1:] == ['']:
             # Strip off a trailing slash
             # This helps both when we are at the root, and when
@@ -130,33 +131,89 @@ def join(base, *args):
     else:
         path = base.split('/')
 
+    if scheme is not None and len(path) >= 1:
+        host = path[:1]
+        # the path should be represented as an abs path.
+        # we know this must be absolute because of the presence of a URL scheme.
+        remove_root = True
+        path = [''] + path[1:]
+    else:
+        # create an empty host, but dont alter the path - this might be a
+        # relative url fragment.
+        host = []
+        remove_root = False
+
     for arg in args:
-        m = _url_scheme_re.match(arg)
-        if m:
+        match = _url_scheme_re.match(arg)
+        if match:
             # Absolute URL
-            scheme = m.group('scheme')
+            scheme = match.group('scheme')
             # this skips .. normalisation, making http://host/../../..
             # be rather strange.
-            path = m.group('path').split('/')
+            path = match.group('path').split('/')
+            # set the host and path according to new absolute URL, discarding
+            # any previous values.
+            # XXX: duplicates mess from earlier in this function.  This URL
+            # manipulation code needs some cleaning up.
+            if scheme is not None and len(path) >= 1:
+                host = path[:1]
+                path = path[1:]
+                # url scheme implies absolute path.
+                path = [''] + path
+            else:
+                # no url scheme we take the path as is.
+                host = []
         else:
-            for chunk in arg.split('/'):
-                if chunk == '.':
-                    continue
-                elif chunk == '..':
-                    if len(path) >= 2:
-                        # Don't pop off the host portion
-                        path.pop()
-                    else:
-                        raise errors.InvalidURLJoin('Cannot go above root',
-                                base, args)
-                else:
-                    path.append(chunk)
+            path = '/'.join(path)
+            path = joinpath(path, arg)
+            path = path.split('/')
+    if remove_root and path[0:1] == ['']:
+        del path[0]
+    if host:
+        # Remove the leading slash from the path, so long as it isn't also the
+        # trailing slash, which we want to keep if present.
+        if path and path[0] == '' and len(path) > 1:
+            del path[0]
+        path = host + path
 
     if scheme is None:
         return '/'.join(path)
     return scheme + '://' + '/'.join(path)
 
 
+def joinpath(base, *args):
+    """Join URL path segments to a URL path segment.
+
+    This is somewhat like osutils.joinpath, but intended for URLs.
+
+    XXX: this duplicates some normalisation logic, and also duplicates a lot of
+    path handling logic that already exists in some Transport implementations.
+    We really should try to have exactly one place in the code base responsible
+    for combining paths of URLs.
+    """
+    path = base.split('/')
+    if len(path) > 1 and path[-1] == '':
+        #If the path ends in a trailing /, remove it.
+        path.pop()
+    for arg in args:
+        if arg.startswith('/'):
+            path = []
+        for chunk in arg.split('/'):
+            if chunk == '.':
+                continue
+            elif chunk == '..':
+                if path == ['']:
+                    raise errors.InvalidURLJoin('Cannot go above root',
+                            base, args)
+                path.pop()
+            else:
+                path.append(chunk)
+    if path == ['']:
+        return '/'
+    else:
+        return '/'.join(path)
+
+
 # jam 20060502 Sorted to 'l' because the final target is 'local_path_from_url'
 def _posix_local_path_from_url(url):
     """Convert a url like file:///path/to/foo into /path/to/foo"""
@@ -171,7 +228,7 @@ def _posix_local_path_to_url(path):
 
     This also handles transforming escaping unicode characters, etc.
     """
-    # importing directly from posixpath allows us to test this 
+    # importing directly from posixpath allows us to test this
     # on non-posix platforms
     return 'file://' + escape(_posix_normpath(
         osutils._posix_abspath(path)))
@@ -191,6 +248,11 @@ def _win32_local_path_from_url(url):
             raise errors.InvalidURL(url, 'Win32 UNC path urls'
                 ' have form file://HOST/path')
         return unescape(win32_url)
+
+    # allow empty paths so we can serve all roots
+    if win32_url == '///':
+        return '/'
+
     # usual local path with drive letter
     if (win32_url[3] not in ('abcdefghijklmnopqrstuvwxyz'
                              'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
@@ -206,18 +268,21 @@ def _win32_local_path_to_url(path):
 
     This also handles transforming escaping unicode characters, etc.
     """
-    # importing directly from ntpath allows us to test this 
+    # importing directly from ntpath allows us to test this
     # on non-win32 platform
     # FIXME: It turns out that on nt, ntpath.abspath uses nt._getfullpathname
     #       which actually strips trailing space characters.
     #       The worst part is that under linux ntpath.abspath has different
     #       semantics, since 'nt' is not an available module.
-    win32_path = osutils._nt_normpath(
-        osutils._win32_abspath(path)).replace('\\', '/')
+    if path == '/':
+        return 'file:///'
+
+    win32_path = osutils._win32_abspath(path)
     # check for UNC path \\HOST\path
     if win32_path.startswith('//'):
         return 'file:' + escape(win32_path)
-    return 'file:///' + win32_path[0].upper() + ':' + escape(win32_path[2:])
+    return ('file:///' + str(win32_path[0].upper()) + ':' +
+        escape(win32_path[2:]))
 
 
 local_path_to_url = _posix_local_path_to_url
@@ -233,12 +298,26 @@ if sys.platform == 'win32':
 
 
 _url_scheme_re = re.compile(r'^(?P<scheme>[^:/]{2,})://(?P<path>.*)$')
+_url_hex_escapes_re = re.compile(r'(%[0-9a-fA-F]{2})')
+
+
+def _unescape_safe_chars(matchobj):
+    """re.sub callback to convert hex-escapes to plain characters (if safe).
+
+    e.g. '%7E' will be converted to '~'.
+    """
+    hex_digits = matchobj.group(0)[1:]
+    char = chr(int(hex_digits, 16))
+    if char in _url_dont_escape_characters:
+        return char
+    else:
+        return matchobj.group(0).upper()
 
 
 def normalize_url(url):
     """Make sure that a path string is in fully normalized URL form.
-    
-    This handles URLs which have unicode characters, spaces, 
+
+    This handles URLs which have unicode characters, spaces,
     special characters, etc.
 
     It has two basic modes of operation, depending on whether the
@@ -257,21 +336,27 @@ def normalize_url(url):
     m = _url_scheme_re.match(url)
     if not m:
         return local_path_to_url(url)
+    scheme = m.group('scheme')
+    path = m.group('path')
     if not isinstance(url, unicode):
         for c in url:
             if c not in _url_safe_characters:
                 raise errors.InvalidURL(url, 'URLs can only contain specific'
                                             ' safe characters (not %r)' % c)
-        return url
-    # We have a unicode (hybrid) url
-    scheme = m.group('scheme')
-    path = list(m.group('path'))
+        path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
+        return str(scheme + '://' + ''.join(path))
 
-    for i in xrange(len(path)):
-        if path[i] not in _url_safe_characters:
-            chars = path[i].encode('utf-8')
-            path[i] = ''.join(['%%%02X' % ord(c) for c in path[i].encode('utf-8')])
-    return str(scheme + '://' + ''.join(path))
+    # We have a unicode (hybrid) url
+    path_chars = list(path)
+
+    for i in xrange(len(path_chars)):
+        if path_chars[i] not in _url_safe_characters:
+            chars = path_chars[i].encode('utf-8')
+            path_chars[i] = ''.join(
+                ['%%%02X' % ord(c) for c in path_chars[i].encode('utf-8')])
+    path = ''.join(path_chars)
+    path = _url_hex_escapes_re.sub(_unescape_safe_chars, path)
+    return str(scheme + '://' + path)
 
 
 def relative_url(base, other):
@@ -283,7 +368,7 @@ def relative_url(base, other):
     dummy, base_first_slash = _find_scheme_and_separator(base)
     if base_first_slash is None:
         return other
-    
+
     dummy, other_first_slash = _find_scheme_and_separator(other)
     if other_first_slash is None:
         return other
@@ -293,6 +378,11 @@ def relative_url(base, other):
     other_scheme = other[:other_first_slash]
     if base_scheme != other_scheme:
         return other
+    elif sys.platform == 'win32' and base_scheme == 'file://':
+        base_drive = base[base_first_slash+1:base_first_slash+3]
+        other_drive = other[other_first_slash+1:other_first_slash+3]
+        if base_drive != other_drive:
+            return other
 
     base_path = base[base_first_slash+1:]
     other_path = other[other_first_slash+1:]
@@ -326,7 +416,7 @@ def _win32_extract_drive_letter(url_base, path):
     # Strip off the drive letter
     # path is currently /C:/foo
     if len(path) < 3 or path[2] not in ':|' or path[3] != '/':
-        raise errors.InvalidURL(url_base + path, 
+        raise errors.InvalidURL(url_base + path,
             'win32 file:/// paths need a drive letter')
     url_base += path[0:3] # file:// + /C:
     path = path[3:] # /foo
@@ -340,7 +430,7 @@ def split(url, exclude_trailing_slash=True):
     :param exclude_trailing_slash: Strip off a final '/' if it is part
         of the path (but not if it is part of the protocol specification)
 
-    :return: (parent_url, child_dir).  child_dir may be the empty string if we're at 
+    :return: (parent_url, child_dir).  child_dir may be the empty string if we're at
         the root.
     """
     scheme_loc, first_path_slash = _find_scheme_and_separator(url)
@@ -405,7 +495,7 @@ def strip_trailing_slash(url):
     if not url.endswith('/'):
         # Nothing to do
         return url
-    if sys.platform == 'win32' and url.startswith('file:///'):
+    if sys.platform == 'win32' and url.startswith('file://'):
         return _win32_strip_local_trailing_slash(url)
 
     scheme_loc, first_path_slash = _find_scheme_and_separator(url)
@@ -450,13 +540,22 @@ def unescape(url):
 # These are characters that if escaped, should stay that way
 _no_decode_chars = ';/?:@&=+$,#'
 _no_decode_ords = [ord(c) for c in _no_decode_chars]
-_no_decode_hex = (['%02x' % o for o in _no_decode_ords] 
+_no_decode_hex = (['%02x' % o for o in _no_decode_ords]
                 + ['%02X' % o for o in _no_decode_ords])
 _hex_display_map = dict(([('%02x' % o, chr(o)) for o in range(256)]
                     + [('%02X' % o, chr(o)) for o in range(256)]))
 #These entries get mapped to themselves
 _hex_display_map.update((hex,'%'+hex) for hex in _no_decode_hex)
 
+# These characters shouldn't be percent-encoded, and it's always safe to
+# unencode them if they are.
+_url_dont_escape_characters = set(
+   "abcdefghijklmnopqrstuvwxyz" # Lowercase alpha
+   "ABCDEFGHIJKLMNOPQRSTUVWXYZ" # Uppercase alpha
+   "0123456789" # Numbers
+   "-._~"  # Unreserved characters
+)
+
 # These characters should not be escaped
 _url_safe_characters = set(
    "abcdefghijklmnopqrstuvwxyz" # Lowercase alpha
@@ -473,16 +572,17 @@ def unescape_for_display(url, encoding):
     This will turn file:// urls into local paths, and try to decode
     any portions of a http:// style url that it can.
 
-    Any sections of the URL which can't be represented in the encoding or 
+    Any sections of the URL which can't be represented in the encoding or
     need to stay as escapes are left alone.
 
     :param url: A 7-bit ASCII URL
     :param encoding: The final output encoding
 
-    :return: A unicode string which can be safely encoded into the 
+    :return: A unicode string which can be safely encoded into the
          specified encoding.
     """
-    assert encoding is not None, 'you cannot specify None for the display encoding.'
+    if encoding is None:
+        raise ValueError('you cannot specify None for the display encoding')
     if url.startswith('file://'):
         try:
             path = local_path_from_url(url)
@@ -522,3 +622,109 @@ def unescape_for_display(url, encoding):
                 # Otherwise take the url decoded one
                 res[i] = decoded
     return u'/'.join(res)
+
+
+def derive_to_location(from_location):
+    """Derive a TO_LOCATION given a FROM_LOCATION.
+
+    The normal case is a FROM_LOCATION of http://foo/bar => bar.
+    The Right Thing for some logical destinations may differ though
+    because no / may be present at all. In that case, the result is
+    the full name without the scheme indicator, e.g. lp:foo-bar => foo-bar.
+    This latter case also applies when a Windows drive
+    is used without a path, e.g. c:foo-bar => foo-bar.
+    If no /, path separator or : is found, the from_location is returned.
+    """
+    if from_location.find("/") >= 0 or from_location.find(os.sep) >= 0:
+        return os.path.basename(from_location.rstrip("/\\"))
+    else:
+        sep = from_location.find(":")
+        if sep > 0:
+            return from_location[sep+1:]
+        else:
+            return from_location
+
+
+def _is_absolute(url):
+    return (osutils.pathjoin('/foo', url) == url)
+
+
+def rebase_url(url, old_base, new_base):
+    """Convert a relative path from an old base URL to a new base URL.
+
+    The result will be a relative path.
+    Absolute paths and full URLs are returned unaltered.
+    """
+    scheme, separator = _find_scheme_and_separator(url)
+    if scheme is not None:
+        return url
+    if _is_absolute(url):
+        return url
+    old_parsed = urlparse.urlparse(old_base)
+    new_parsed = urlparse.urlparse(new_base)
+    if (old_parsed[:2]) != (new_parsed[:2]):
+        raise errors.InvalidRebaseURLs(old_base, new_base)
+    return determine_relative_path(new_parsed[2],
+                                   join(old_parsed[2], url))
+
+
+def determine_relative_path(from_path, to_path):
+    """Determine a relative path from from_path to to_path."""
+    from_segments = osutils.splitpath(from_path)
+    to_segments = osutils.splitpath(to_path)
+    count = -1
+    for count, (from_element, to_element) in enumerate(zip(from_segments,
+                                                       to_segments)):
+        if from_element != to_element:
+            break
+    else:
+        count += 1
+    unique_from = from_segments[count:]
+    unique_to = to_segments[count:]
+    segments = (['..'] * len(unique_from) + unique_to)
+    if len(segments) == 0:
+        return '.'
+    return osutils.pathjoin(*segments)
+
+
+
+def parse_url(url):
+    """Extract the server address, the credentials and the path from the url.
+
+    user, password, host and path should be quoted if they contain reserved
+    chars.
+
+    :param url: an quoted url
+
+    :return: (scheme, user, password, host, port, path) tuple, all fields
+        are unquoted.
+    """
+    if isinstance(url, unicode):
+        raise errors.InvalidURL('should be ascii:\n%r' % url)
+    url = url.encode('utf-8')
+    (scheme, netloc, path, params,
+     query, fragment) = urlparse.urlparse(url, allow_fragments=False)
+    user = password = host = port = None
+    if '@' in netloc:
+        user, host = netloc.rsplit('@', 1)
+        if ':' in user:
+            user, password = user.split(':', 1)
+            password = urllib.unquote(password)
+        user = urllib.unquote(user)
+    else:
+        host = netloc
+
+    if ':' in host and not (host[0] == '[' and host[-1] == ']'): #there *is* port
+        host, port = host.rsplit(':',1)
+        try:
+            port = int(port)
+        except ValueError:
+            raise errors.InvalidURL('invalid port number %s in url:\n%s' %
+                                    (port, url))
+    if host != "" and host[0] == '[' and host[-1] == ']': #IPv6
+        host = host[1:-1]
+
+    host = urllib.unquote(host)
+    path = urllib.unquote(path)
+
+    return (scheme, user, password, host, port, path)
